/* main.c */

/* Dummy main program for calling the FOA routines. */

/* Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)] */

#include <foa.h>
#include <rsta_types.h>
#include <rsta_modules.h>
#include <stdio.h>

extern struct xvimage *readimage();
extern void draw_rectangles(struct xvimage     *image,
			    detection_desc_t   *detects,
			    int                 num_detects,
			    int                 verbose);

extern int writeimage(char *, struct xvimage *);
extern int freeimage(struct xvimage *);


main(int argc, char *argv[])
{
  struct xvimage *image;
  float range = 800.0;
  float hfov = 4.0;
  float vfov = 3.0;
  int num_detects, verbose = 0;
  detection_desc_t *detects;
  environ_t *environ;
  float TGT_PIX_PCNT = COLOR_TARGET_DETECT_TGT_PIX_PCNT;

  /* status_sensor_state_msg *sensor; */
  
  /* Parse command line. */
  if (argc < 2 || argc > 5) {
    fprintf(stderr, 
	    "Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)] \n"); 
    exit(1);
   }
  else if (argc == 3) {
    if (argv[2][0] == '-' && argv[2][1] == 'v')
      verbose = 1;
    else if (argv[2][0] == '-' && argv[2][1] == 'V')
      verbose = 2;
    else {
      fprintf(stderr, 
	      "Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)]\n");
      exit(1);
    }
  }
  else if (argc == 4) {
    if (argv[2][0] == '-' && argv[2][1] == 'p')
      TGT_PIX_PCNT = atof(argv[3]);
    else {
      fprintf(stderr, 
	      "Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)]\n");
      exit(1);
    }
  }
  else  if (argc == 5) {
    if (argv[2][0] == '-' && argv[2][1] == 'p')
      TGT_PIX_PCNT = atof(argv[3]);
    else {
      fprintf(stderr, 
	      "Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)]\n");
      exit(1);
    }
    if (argv[4][0] == '-' && argv[4][1] == 'v')
      verbose = 1;
    else if (argv[4][0] == '-' && argv[4][1] == 'V')
      verbose = 2;
    else {
      fprintf(stderr, 
	      "Usage: foa imagein [-p# (tgt pix percent : float)] [-vV (verbose Very_verbose)]\n");
      exit(1);
    }
  }
  
  /* Set verbosity. */
  color_target_detect_set_verbose(verbose);

   /* Perform initialization. */
  if (color_target_detect_initialize())
    {fprintf(stderr, "Initialization failed\n"); exit(1);}

  /* Now the "color-target-detection executive" would obtain an image from the digitizer
     and convert it to viff format, ie., an `xvimage'.  Here, we're going
     to simulate that by reading the image from a file. */
  image = readimage(argv[1]);  /* Khoros routine. */
  environ = (environ_t *)malloc(sizeof(*environ));
  environ->datetime2[0] = 0;
  environ->month_day[0] = 0;
  sprintf(environ->date_string, "Happy Bird-watching!");

  /* Call FOA routine inside here. */ 

  if (color_target_detect_execute(image, hfov, vfov, range, 
				  environ, &num_detects, &detects, TGT_PIX_PCNT))
    {fprintf(stderr, "Execution failed\n"); exit(1);}

  /* Free memory. */
  if (color_target_detect_finish())
    {fprintf(stderr, "Termination failed\n"); exit(1);}

  /* Draw the bounding rectangles on the image. */
  draw_rectangles(image, detects, num_detects, verbose);

  /* Save the results in a file. */
  writeimage("result.xv", image);
  freeimage(image);
  free(detects);

  return (0);
}

