/*------------------------------------------------------------------------------
$Id: rsta_types.h,v 2.9 1995/04/07 20:11:03 severson Exp $

     This software was developed by Martin Marietta Corporation under the
     DARPA-sponsored UGV Demo II program, Contract DAAH01-92-C-R101.
     Distribution is restricted to the government and contractors under the
     UGV Demo II program.

	FILE:   RSTA_types.h

	AUTHOR: Bill Severson 7-2577
	        Tue Oct 19 12:51:31 MDT 1993  file created

	DESCRIPTION:  supporting type definitions for rsta
$Log: rsta_types.h,v $
 * Revision 2.9  1995/04/07  20:11:03  severson
 * *** empty log message ***
 *
 * Revision 2.8  1995/02/25  00:31:13  severson
 * removed unneeded include of vcs_types.h
 *
 * Revision 2.7  1995/02/22  23:29:45  severson
 * minor mods for confirm task.
 *
 * Revision 2.6  1995/02/20  19:48:54  severson
 * new result_code value
 *
 * Revision 2.5  1995/02/18  00:23:08  severson
 * minor target resp t change, tgt desc change.
 *
 * Revision 2.4  1995/02/14  20:49:18  allmen
 * Put horizon_pt_t back in! Who deleted it???
 *
 * Revision 2.3  1995/02/07  21:11:32  lee
 * *** empty log message ***
 *
 * Revision 2.2  1995/02/06  18:20:43  severson
 * adds for new RstaATR module
 *
 * Revision 1.4  1995/01/20  00:02:22  severson
 * *** empty log message ***
 *
 * Revision 1.3  1995/01/19  17:25:46  severson
 * updated detection_desc_t, added ident_desc_t
 *
 * Revision 1.2  1994/05/24  21:02:12  severson
 * *** empty log message ***
 *

------------------------------------------------------------------------------*/

#ifndef __RSTA_TYPES_H__
#define __RSTA_TYPES_H__

/*
 *----------------------- PUBLIC  DECLARATIONS ---------------------------------
 */

/*------------------------------------------------------------------------------
	CONSTANTS:
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
        ENUMERATIONS:
------------------------------------------------------------------------------*/


/* task type definitions.  used in RSTA_task_status_resp_t,
   for task_type (in the status structure). */
typedef enum {
    RSTA_TARGET_SEARCH = 0,
    RSTA_TERRAIN_RECON,
    RSTA_PANORAMA,
    RSTA_ACQUIRE_TARGET,
    RSTA_LOOK,
    RSTA_FIRE_CONTROL,
    RSTA_MANUAL_CONTROL,
    RSTA_STATUS,
    RSTA_TARGET_CONFIRM
} task_type_t;

/* task status definitions.  used in RSTA_task_status_resp_t,
   for task_status (in the status structure). */
typedef enum {
    TASK_PAUSED = 0,             /* has rec'd a pause command from ows.  */
    TASK_ACTIVE,                 /* task is running                      */
    TASK_AWAITING_RESOURCES,     /* task is ready to run, waiting for    */
                                 /* some other task to complete.         */
    TASK_NEW,                    /* newly created task, not yet init'd   */
    TASK_COMPLETED,              /* task completed normally.             */
    TASK_TERMINATED,             /* task has rec'd a terminate command   */
    TASK_ERROR_INTERNAL,         /* completed, with some internal error, */
                                 /* likely a coding problem.             */
    TASK_ERROR_EXTERNAL          /* completed, with some external error, */
                                 /* likely a communications problem.     */
} task_status_t;

/* task control definitions.  used in RSTA_task_control_cmd_t,
   the control_cmd field, refering to the task_id_to_control field. */
typedef enum {
    TASK_PAUSE = 0,
    TASK_RESUME,
    TASK_TERMINATE,
    TASK_TERMINATE_ALL,
    TASK_PAUSE_ALL
} control_cmd_t;

/* area description types, the different ways of defining an area
   in an area_desc_t  */
typedef enum {
    AZIMUTH_RANGE = 0,
    MAP_REGION
} area_type_t;

/* location description types, the different ways of specifying a
   location in a loc_desc_t */
typedef enum {
    MAP_LOCATION = 0,
    ANGLE_DIRECTION,
    TARGET_ID
} loc_type_t;

/* another location description type, for view sequences. */
typedef enum {
    UTM_3D_LOCATION = 0,
    AZ_EL_ANGLES
} loc2_type_t;

/* coordinate system possibilities for azimuth ranges. */
typedef enum {
    RSTA_VEHICLE_COORDS = 0,
    RSTA_WORLD_COORDS
} coord_sys_t;

/* result codes for atr algorithms.  The 'no target information' value
   specifies that the given data structure is not filled out.
 */
typedef enum {
    TARGET_NOT_FOUND = 0,
    TARGET_DETECTED,
    TARGET_CLASSIFIED,
    TARGET_RECOGNIZED,
    TARGET_IDENTIFIED,
    NO_TARGET_INFORMATION
} result_code_t;

/* sensor id's, for the manual grab image command, 
   and sensor spec for sequences */
typedef enum {
    RSTA_SENSOR_COLOR_CCD = 0,
    RSTA_SENSOR_FLIR,
    RSTA_SENSOR_LADAR,
    RSTA_BOTH_FLIR_AND_COLOR
} rsta_sensor_id_t;

/* focal lengths, for the case of dual fov flir.  */
typedef enum {
    WIDE_FOV,
    NARROW_FOV
} flir_focal_len_t;

/* IFF classifications... */
typedef enum {
    FRIENDLY = 0,
    FOE,
    UNKNOWN
} iff_t;


/* formation related enumerations */
typedef enum {
    LINE_LEFT = 0,
    LINE_RIGHT,
    WEDGE_LEFT,
    WEDGE_RIGHT,
    COLUMN
} formation_type_t;

typedef enum {
    UNIT_CENTER = 0, 
    LEADER
} formation_ref_t;

typedef enum {
    CAN_ACCEPT = 0,
    CANNOT_ACCEPT,
    CANNOT_ACCEPT_I_AM_BUSY,
    CANNOT_ACCEPT_AREA_NOT_IN_VIEW,
    COMMANDED_HANDOFF_SUCCESS,
    COMMANDED_HANDOFF_FAILURE,
    COMMANDED_HANDBACK_COMPLETE,
    COMMANDED_HANDBACK_FAILURE
} handoff_status_t;

typedef enum {
    CAN_CONFIRM = 0,
    CANNOT_CONFIRM,
    CANNOT_CONFIRM_I_AM_BUSY,
    CANNOT_CONFIRM_NOT_IN_VIEW,
    CANNOT_CONFIRM_NOT_IN_RANGE,
    TARGET_CONFIRMED,
    TARGET_NOT_CONFIRMED
} confirm_status_t;

/*------------------------------------------------------------------------------
	STRUCTURES:
------------------------------------------------------------------------------*/

/* Type of a point in a computed horizon image */
typedef struct {
    int x; int y;                         /* Location in an image */
    int confidence;                       /* Confidence of horizon point */
} horizon_pt_t;
 
typedef struct map_loc_struct {
    float             x;        /* UTM coords. x is north. */
    float             y;        /* UTM coords. y is east.  */
} map_loc_t;


typedef struct image_loc_struct {
    int               x;        /* image coords.  x traverses columns. */
    int               y;        /* image coords.  y traverses rows. */
} image_loc_t;


typedef struct image_size_struct {                                          
    int               x;        /* image coords.  extent in pixels (cols) */
    int               y;        /* image coords.  extent in pixels (rows) */
} image_size_t;                                                             


typedef struct image_velocity_struct {
    float x;                    /* x velocity in image coords. */
                                /* x traverses columns.        */
    float y;                    /* y velocity in image coords. */
                                /* y traverses columns.        */
} image_velocity_t;


/* note: pan_tilt_angle_t is defined in SSV/include/vcs_types.h */


typedef struct pan_tilt_rate_struct {
    float pan_rate;
    float tilt_rate;
} pan_tilt_rate_t;


typedef struct point_struct {
    float             x;        /* generic position. */
    float             y;
    float             z;
} point_3d_t;


typedef struct vector_dir_struct {
    float             x;        /* generic direction. */
    float             y;
    float             z;
} vector_3d_t;


typedef struct angle_dir_struct {
    float             azimuth;        /* degrees.  0 is forward(or North)*/
                                      /*  + azimuth is to the right      */
    float             elevation;      /* degrees.  0 is level, + is up.  */
    coord_sys_t       coord_sys;      /* for the angle direction; one    */
                                      /* RSTA_VEHICLE_COORDS or          */
                                      /* RSTA_WORLD_COORDS.              */
} angle_dir_t;


typedef struct azimuth_range_struct { 
    float             left_azimuth;   /* degrees.                        */
    float             right_azimuth;  /* degrees.                        */
    coord_sys_t       coord_sys;      /* for the azimuth range, one of   */
                                      /* VEHICLE or WORLD coordinates.   */
} azimuth_range_t;


typedef struct loc_desc_struct {
    loc_type_t        loc_type;       /* one of MAP_LOCATION, TARGET_ID, */
				      /* ANGLE_DIRECTION                 */
    map_loc_t         map_loc;        /* location in world coords.       */
    int               target_id;      /* location of the given target    */
    angle_dir_t       angle_dir;      /* az, el pair                     */
} loc_desc_t;


typedef struct loc2_desc_struct {
    loc2_type_t       loc2_type;      /* one of UTM_3D_LOCATION, or      */
				      /* AL_EL_ANGLES                    */
    point_3d_t        point_3d;       /* location in world coords.       */
    angle_dir_t       angle_dir;      /* az, el angle pair               */
} loc2_desc_t;


typedef struct environ_struct {
    /* various versions of the current date and time.                   */
    int datetime2[6];                 /* year, julian_day, hour, min,   */
                                      /* sec, msec                      */
    int month_day[2];                 /* month, day                     */
    char date_string[40];             /* string version of date, i.e.   */
                                      /* Sun Sep 16 01:03:52 1998\n\0   */

    /* ... */
} environ_t;


typedef struct detection_desc_struct {

    result_code_t     result_code;        /* detected, or not detected   */
    float             confidence;         /* 0..1                        */
    int               num_other_SSV_confirmations;

    /* FLIR_ quantities are in FLIR image pixel units. (BB=bounding box) */
    image_loc_t       FLIR_loc;           /* BB location (upper left)    */
    image_size_t      FLIR_size;          /* BB size                     */
    image_loc_t       FLIR_centroid;      /* blob centroid               */
    int               FLIR_area;          /* blob area in pixels         */

    /* CCD_ quantities are in color image pixel units.                   */
    image_loc_t       CCD_loc;            /* BB location (upper left)    */
    image_size_t      CCD_size;           /* BB size                     */
    image_loc_t       CCD_centroid;       /* blob centroid               */
    int               CCD_area;           /* blob area in pixels         */

    /* unique target id is assigned by RstaVeh, loc & range are set      */
    /* using active ranging or ray tracing with elevation map.           */
    int               target_id;          /* a new or existing ID #      */
					  /* for the target database     */
    map_loc_t         target_loc;         /* world location of target    */
    float             range;              /* range to target             */

} detection_desc_t;


typedef struct ident_desc_struct {

    result_code_t     result_code;        /* no tgt, detected, class,    */
					  /* recognized, identified.     */
    char              type[128];          /* detailed info. ident str.   */

    float             pose[3];		  /* orientation.                */
                                          /* units, conventions, TBD.    */
    float             confidence;         /* 0..1                        */

} ident_desc_t;


/* convenient grouping of target description data. */
typedef struct tgt_desc_struct {
    detection_desc_t  detect;
    ident_desc_t      ident;
} tgt_desc_t;


typedef struct task_summary_status_struct {
    int               task_instance_id;   /* unique task (command) id    */
    task_type_t       task_type;          /* one of search, recon..      */
    task_status_t     task_status;        /* one of active, paused,      */
    float             task_percent_complete;  /*  0 to 100               */
} task_summary_status_t;


typedef struct sensor_desc_struct {
    rsta_sensor_id_t   sensor_id;         /* one of flir, color, ladar   */
    float              CCD_zoom;          /* for the case of color.      */
    float              CCD_focus;         /* for the case of color.      */
    flir_focal_len_t   focal_length;      /* for the case of flir.       */
} sensor_desc_t;


typedef struct view_desc_struct {
    loc2_desc_t       location;           /* where to direct sensor      */
    float             angular_FOV_deg;    /* azimuth range about dir     */
    sensor_desc_t     sensor;             /* sensor to use               */
    int               weight;             /* focus weight for this view  */
} view_desc_t;


typedef struct rsta_formation_struct {
    formation_type_t     formation_type;  /* LINE_LEFT, WEDGE_LEFT, etc. */
    formation_ref_t      formation_ref;   /* UNIT_CENTER, or LEADER      */
    float                spacing;         /* space btwn vehicles, meters */
    int                  num_vehicles;    /* # of vehicles in formation  */
    int                  vehicle_ids[4];  /* vehicle ID's,(def in SSV.h) */
    int                  stations[4];     /* where vehicles are in form. */ 
} rsta_formation_t;


#endif __RSTA_TYPES_H__
