/* =============================================================
   mdtrain: Gathers training data for building a 
            multivariate decision tree for classification of
	    a color image into target and non-target regions.

	    Usage is as follows:
	      mdtrain training_image(input) feature_file(output)

	      'training_image' is the image from which the training
	      data (pixels) is to be extracted.  There are two types of 
	      training data -- positive (class 1), representing targets,
	      and negative (class 0), representing non-target regions.

	      After extraction of sub-regions, a 'feature-file' is built
	      by the program, which stores the training data in a form
	      usable by the routine which builds the decision tree.

   Shashi Buluswar
   Computer Vision Laboratory
   Dept of Computer Science
   University of Massachusetts
   Amherst, MA  01003

   Copyright 1995, University of Massachusetts - Amherst

   =============================================================== */

#define MAIN
#include <mdtrain.h>


void CreateFeatFromImg(char *img, int img_class, char *featFile)

{
  struct xvimage *image;
  unsigned char *R, *G, *B, *r, *g, *b;
  int i, w, h, n, iR, iG, iB;
  FILE *featFP;

  if ((featFP = fopen (featFile, "a")) == NULL) {
    printf ("error opening feat file\n");
    exit (1);
  }
  image = readimage(img);  /* Khoros routine. */

  w = image->row_size; h = image->col_size;
  n = w * h;

  R = (unsigned char *)image->imagedata;
  G = R + n;
  B = G + n;

  for (i = 0, r = R, g = G, b = B; i < n; i++, r++, g++, b++) {
    iR = *r; iG = *g; iB = *b;
    fprintf (featFP, "%d %d %d %d\n", img_class, iR, iG, iB);
  }

}


main (int argc, char *argv[])

{
  char *IN_IMG, *FEAT_FILE;
  int CLASS;

  /* === show usage === */
  if (argc != 4) {
    fprintf(stdout, "\nusage: mdtrain image.viff class FeatureFile\n");
    return (-1);
  }

  IN_IMG = argv[1];
  CLASS = atoi(argv[2]);
  FEAT_FILE = argv[3];

  /* === create ascii RGB files === */
  CreateFeatFromImg(IN_IMG, CLASS, FEAT_FILE);

  /*printf ("done...\n");*/
}


